/*
 * SienaGauge.java
 *
 * Created on November 5, 2001, 4:02 PM
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.CreationFinishedEvent;
import java.net.*;
import java.util.*;
/**
 *
 * @author  Bradley Schmerl
 * @version 1.1
 */
public class SienaGauge implements edu.cmu.cs.able.gaugeInfrastructure.GaugeControl, Runnable {
    
    /** The bus to report to.
     */
    protected GaugeReportingBus gaugeBus = new SienaGaugeReportingBus (); // The bus to report to
    /** The type of the gauge (should be overridden by extending gauges)
     */
    protected String gaugeType = "edu.cmu.cs.able.gaugeInfrastructure.Siena.genericGauge"; // The type of the gauge
    /** The ID of the gauge manager controlling this gauge.
     */
    protected String myManagerID = null;
    /** Indicates whether the reporting thread should report values.
     */
    protected boolean doReporting = false;
    
    private Thread reportingThread = null;
    /** How long (milliseconds) to sleep before reporting another value.
     */
    protected int sleepTime = 1000;
    
    /** The parameters used to setup the gauge.
     */
    protected StringPairVector setupParams;
    /** The mappings of values to properties.
     */
    protected Hashtable mappings = new Hashtable ();
    
    GaugeID gaugeID;
    
    /** Creates new SienaGauge
     * @param modelName The name of the model that the gauge is attached to.
     * @param modelType The type of the model that the gauge is attached to.
     * @param gaugeName The name of the new gauge.
     * @param creatorID The gauge manager that created the gauge.
     */
    public SienaGauge (String modelName, String modelType, String gaugeName, String creatorID) {
        gaugeID = new GaugeID ();
        gaugeID.uid = edu.cmu.cs.able.gaugeInfrastructure.util.Global.generateUniqueID ();
        gaugeID.modelName = modelName;
        gaugeID.modelType = modelType;
        gaugeID.gaugeName = gaugeName;
        gaugeID.gaugeType = gaugeName;
        myManagerID = creatorID;
        
        setupParams = new StringPairVector ();
        StringPairVector mappingsVec = new StringPairVector ();
        // Register with the bus, and wait for my my setup params
        ((SienaGaugeReportingBus )gaugeBus).registerGauge (this, setupParams, mappingsVec, true, myManagerID);
        
        this.xferMappings (mappingsVec);
        this.start ();
    }
    
    private void xferMappings (StringPairVector mappingsVec) {
        
        for (int i = 0; i < mappingsVec.size (); i++) {
            mappings.put (mappingsVec.nameAt (i), mappingsVec.valueAt (i));
        }
        
    }
    
    /** Creates a new gauge if all information is known.
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager creating the gauge.
     * @param setupParams The parameters with which to setup the gauge.
     * @param mappings The value->property mappings within the model the gauge is attached to.
     */
    public SienaGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        this.gaugeID = gaugeID;
        gaugeID.uid = edu.cmu.cs.able.gaugeInfrastructure.util.Global.generateUniqueID ();
        myManagerID = creatorID;
        
        this.setupParams = setupParams;
        
        ((SienaGaugeReportingBus )gaugeBus).registerGauge (this, this.setupParams, mappings, mappings.isEmpty () || setupParams.isEmpty (), myManagerID);
        this.xferMappings (mappings);
        this.start ();
    }
    
    /** Creates and starts the reporting thread.
     */
    public void start () {
        if (reportingThread == null) {
            reportingThread = new Thread (this, "reporting");
            reportingThread.start ();
        }
    }
    
    /** This is the reporting thread. It calls <i>reportNewValue</i> which
     * should be overridden by extenders of this gauge.
     */
    public void run () {
        Thread myThread = Thread.currentThread ();
        while (reportingThread == myThread) {
            if (doReporting) this.reportNewValue ();
            try {
                Thread.sleep (sleepTime);
            }
            catch (InterruptedException e) {
            }
        }
    }
    
    /** Stops gauge reporting.
     */
    public void stop () {
        reportingThread = null;
    }
    
    /** Returns the gauge ID of this gauge.
     * @return
     */
    public GaugeID getGaugeID () {
        return gaugeID;
    }
    
    /** Sets the GaugeID of this gauge.
     */
    public void setGaugeID (GaugeID id) {
        gaugeID = id;
    }
    
    // These methods must be overridden by particular gauges
    
    /** Reports a new value. This should be overridden by extenders of this gauge.
     */
    public void reportNewValue () {
        System.out.println (gaugeID.toString () + " reporting a value");
    }
    
    /** Configures the gauge.
     * @param configParams The configuration parameters for this gauge.
     * @return
     */
    public boolean configure (StringPairVector configParams) {
        System.out.println (gaugeID.toString () + " is configured");
        return false;
    }
    
    /** Returns all the values that this gauge reports.
     * @param values
     * @return
     */
    public boolean queryAllValues (GaugeValueVector values) {
        System.out.println (gaugeID.toString () + " is queried for all values");
        return false;
    }
    
    /** Returns the state of this gauge.
     * @param setupParams
     * @param configParams
     * @param mappings
     * @return
     */
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        System.out.println (gaugeID.toString () + " has been queried for state");
        return false;
    }
    
    /** Returns the value of a particular value name
     * @param valueName
     * @return
     */
    public String queryValue (String valueName) {
        System.out.println (gaugeID.toString () + " has been queried for a value");
        return "dummy value";
        
    }
    
    /** Processes a delete message from the gauge manager. This method follows the process for
     * deleting a gauge, i.e.:<br>
     *   report the fact that the gauge was deleted to the gauge bus<br>
     *   stop reporting values<br>
     *   don't listen to any more events<br>
     * It should be assumed by the gauge that it will be destroyed after this method is called
     * (destroyed could mean that the JVM running this gauge is stopped)
     * @param gaugeMgrID The ID of the gauge manager requesting deletion.
     */
    public void processDelete (String gaugeMgrID) {
        finalizeDeletion (gaugeMgrID);
    }
    
    protected void finalizeDeletion (String gaugeMgrID) {
        DeletedEvent event = new DeletedEvent (gaugeID);
        event.gaugeMgrID = gaugeMgrID;
        event.status = true;
        gaugeBus.reportDeleted (event);
        reportingThread = null;
        ((SienaGaugeReportingBus )gaugeBus).sienaBus.siena.shutdown ();
        //((SienaGaugeReportingBus )gaugeBus).sienaBus.unregisterGauge (this);
    }
    
    public void finalizeCreation (String gaugeMgrID, boolean successful) {
        CreationFinishedEvent event = new CreationFinishedEvent (this.getGaugeID ());
        event.gaugeMgrID = gaugeMgrID;
        event.status = successful;
        ((SienaGaugeReportingBus )gaugeBus).sienaBus.publishAnnouncement (event);
        
        if (successful) {
            CreatedEvent ce = new CreatedEvent (this.getGaugeID ());
            ce.status = successful;
            ce.gaugeMgrID = gaugeMgrID;
            ((SienaGaugeReportingBus )gaugeBus).sienaBus.publishAnnouncement (ce);
        }
    }
}